searchData={"items":[{"type":"module","title":"SafeURL","doc":"`SafeURL` is library for mitigating Server Side Request\nForgery vulnerabilities in Elixir. Private/reserved IP\naddresses are blocked by default, and users can add\nadditional CIDR ranges to the blocklist, or alternatively\nallow specific CIDR ranges to which the application is\nallowed to make requests.\n\nYou can use `allowed?/2` or `validate/2` to check if a\nURL is safe to call.","ref":"SafeURL.html"},{"type":"module","title":"Examples - SafeURL","doc":"iex> SafeURL.allowed?(\"https://includesecurity.com\")\n    true\n\n    iex> SafeURL.validate(\"http://google.com/\", schemes: ~w[https])\n    {:error, :unsafe_scheme}\n\n    iex> SafeURL.validate(\"http://230.10.10.10/\")\n    {:error, :unsafe_reserved}\n\n    iex> SafeURL.validate(\"http://230.10.10.10/\", block_reserved: false)\n    :ok\n\n    # If HTTPoison is available:\n\n    iex> SafeURL.HTTPoison.get(\"https://10.0.0.1/ssrf.txt\")\n    {:error, :unsafe_reserved}\n\n    iex> SafeURL.HTTPoison.get(\"https://google.com/\")\n    {:ok, %HTTPoison.Response{...}}","ref":"SafeURL.html#module-examples"},{"type":"module","title":"Options - SafeURL","doc":"`SafeURL` can be configured to customize and override\nvalidation behaviour by passing the following options:\n\n  * `:block_reserved` - Block reserved/private IP ranges.\n    Defaults to `true`.\n\n  * `:blocklist` - List of CIDR ranges to block. This is\n    additive with `:block_reserved`. Defaults to `[]`.\n\n  * `:allowlist` - List of CIDR ranges to allow. If\n    specified, blocklist will be ignored. Defaults to `[]`.\n\n  * `:schemes` - List of allowed URL schemes. Defaults to\n    `[\"http, \"https\"]`.\n\n  * `:dns_module` - Any module that implements the\n    `SafeURL.DNSResolver` behaviour. Defaults to `DNS` from\n    the `:dns` package.\n\n  * `:detailed_error` - Return specific error if validation fails. If set to\n    `false`, `validate/2` will return `{:error, :restricted}` regardless of\n    the reason. Defaults to `true`.\n\n\nIf `:block_reserved` is `true` and additional hosts/ranges\nare supplied with `:blocklist`, both of them are included in\nthe final blocklist to validate the address. If allowed\nranges are supplied with `:allowlist`, all blocklists are\nignored and any hosts not explicitly declared in the allowlist\nare rejected.\n\nThese options can be set globally in your `config.exs` file:\n\n    config :safeurl,\n      block_reserved: true,\n      blocklist: ~w[100.0.0.0/16],\n      schemes: ~w[https],\n      dns_module: MyCustomDNSResolver\n\nOr they can be passed to the function directly, overriding any\nglobal options if set:\n\n    iex> SafeURL.validate(\"http://10.0.0.1/\", block_reserved: false)\n    :ok\n\n    iex> SafeURL.validate(\"https://app.service/\", allowlist: ~w[170.0.0.0/24])\n    :ok\n\n    iex> SafeURL.validate(\"https://app.service/\", blocklist: ~w[170.0.0.0/24])\n    {:error, :unsafe_blocklist}","ref":"SafeURL.html#module-options"},{"type":"function","title":"SafeURL.allowed?/2","doc":"Validate a string URL against a blocklist or allowlist.\n\nThis method checks if a URL is safe to be called by looking at\nits scheme and resolved IP address, and matching it against\nreserved CIDR ranges, and any provided allowlist/blocklist.\n\nReturns `true` if the URL meets the requirements,\n`false` otherwise.","ref":"SafeURL.html#allowed?/2"},{"type":"function","title":"Examples - SafeURL.allowed?/2","doc":"iex> SafeURL.allowed?(\"https://includesecurity.com\")\n    true\n\n    iex> SafeURL.allowed?(\"http://10.0.0.1/\")\n    false\n\n    iex> SafeURL.allowed?(\"http://10.0.0.1/\", allowlist: ~w[10.0.0.0/8])\n    true","ref":"SafeURL.html#allowed?/2-examples"},{"type":"function","title":"Options - SafeURL.allowed?/2","doc":"See [`Options`](#module-options) section above.","ref":"SafeURL.html#allowed?/2-options"},{"type":"function","title":"SafeURL.validate/2","doc":"Alternative method of validating a URL, returning result tuple instead\nof booleans.\n\nThis calls `allowed?/2` underneath to check if a URL is safe to\nbe called. If it is, it returns `:ok`, otherwise an error tuple with a\nspecific reason. If `:detailed_error` is set to `false`, the error is always\n`{:error, :restricted}`.","ref":"SafeURL.html#validate/2"},{"type":"function","title":"Examples - SafeURL.validate/2","doc":"iex> SafeURL.validate(\"https://includesecurity.com\")\n    :ok\n\n    iex> SafeURL.validate(\"http://10.0.0.1/\")\n    {:error, :unsafe_reserved}\n\n    iex> SafeURL.validate(\"http://10.0.0.1/\", allowlist: ~w[10.0.0.0/8])\n    :ok","ref":"SafeURL.html#validate/2-examples"},{"type":"function","title":"Options - SafeURL.validate/2","doc":"See [`Options`](#module-options) section above.","ref":"SafeURL.html#validate/2-options"},{"type":"type","title":"SafeURL.error/0","doc":"","ref":"SafeURL.html#t:error/0"},{"type":"behaviour","title":"SafeURL.DNSResolver","doc":"In some cases you might want to use a custom strategy\nfor DNS resolution. You can do so by passing your own\nimplementation of `SafeURL.DNSResolver` in the global\nor local config.\n\nBy default, the `DNS` package is used for resolution,\nbut you can replace it with a wrapper that uses\ndifferent configuration or a completely different\nimplementation altogether.","ref":"SafeURL.DNSResolver.html"},{"type":"behaviour","title":"Use-cases - SafeURL.DNSResolver","doc":"* Using a specific DNS server\n  * Avoiding network access in specific environments\n  * Mocking DNS resolution in tests","ref":"SafeURL.DNSResolver.html#module-use-cases"},{"type":"behaviour","title":"Usage - SafeURL.DNSResolver","doc":"Start by creating a module that implements the\n`DNSResolver` behaviour. Currently, this means adding\nonly one `c:resolve/1` callback that takes a host and\nreturns a list of resolved IPs.\n\nAs an example, suppose you wanted to use\n[Cloudflare's DNS](https://1.1.1.1/dns/), you can do\nthat by wrapping `DNS` with your own settings in a new\nmodule:\n\n    defmodule CloudflareDNS do\n      @behaviour SafeURL.DNSResolver\n\n      @impl true\n      def resolve(domain) do\n        DNS.resolve(domain, :a, {\"1.1.1.1\", 53}, :udp)\n      end\n    end\n\nTo use it, simply pass it in the global config:\n\n    config :safeurl, dns_module: CloudflareDNS\n\nYou can also directly set the `:dns_module` in method options:\n\n    SafeURL.allowed?(\"https://example.com\", dns_module: CloudflareDNS)","ref":"SafeURL.DNSResolver.html#module-usage"},{"type":"behaviour","title":"Testing - SafeURL.DNSResolver","doc":"This is especially useful in tests where you want to\nensure your HTTP Client wrapper with `SafeURL` is\nworking as expected.\n\nYou can override the `:dns_module` config to ensure\na specific IP is resolved for a domain or no network\nrequests are made:\n\n    defmodule TestDNSResolver do\n      @behaviour SafeURL.DNSResolver\n\n      @impl true\n      def resolve(\"google.com\"), do: {:ok, [{192, 168, 1, 10}]}\n      def resolve(\"github.com\"), do: {:ok, [{192, 168, 1, 20}]}\n      def resolve(_domain),      do: {:ok, [{192, 168, 1, 99}]}\n    end","ref":"SafeURL.DNSResolver.html#module-testing"},{"type":"callback","title":"SafeURL.DNSResolver.resolve/1","doc":"","ref":"SafeURL.DNSResolver.html#c:resolve/1"},{"type":"module","title":"SafeURL.HTTPoison","doc":"A utility module that should be a drop-in replacement for `HTTPoison`. Only\nsupports `get/3`.","ref":"SafeURL.HTTPoison.html"},{"type":"function","title":"SafeURL.HTTPoison.get/3","doc":"Validate a URL and execute a GET request using `HTTPoison`.\n\nIf the URL is safe, this function will execute the request using\n`HTTPoison`, returning the result directly. Otherwise, it will\nreturn error.\n\n`headers` and `options` will be passed directly to\n`HTTPoison` when the request is executed.","ref":"SafeURL.HTTPoison.html#get/3"},{"type":"function","title":"Examples - SafeURL.HTTPoison.get/3","doc":"iex> SafeURL.HTTPoison.get(\"https://10.0.0.1/ssrf.txt\")\n    {:error, :unsafe_reserved}\n\n    iex> SafeURL.HTTPoison.get(\"https://google.com/\")\n    {:ok, %HTTPoison.Response{...}}","ref":"SafeURL.HTTPoison.html#get/3-examples"},{"type":"module","title":"SafeURL.TeslaMiddleware","doc":"Tesla middleware for validating URLs.","ref":"SafeURL.TeslaMiddleware.html"},{"type":"module","title":"Examples - SafeURL.TeslaMiddleware","doc":"iex> Tesla.client([SafeURL.TeslaMiddleware]) |> Tesla.get(\"http://localhost/\")\n    {:error, :unsafe_reserved}","ref":"SafeURL.TeslaMiddleware.html#module-examples"},{"type":"function","title":"SafeURL.TeslaMiddleware.call/3","doc":"","ref":"SafeURL.TeslaMiddleware.html#call/3"},{"type":"extras","title":"SafeURL","doc":"# SafeURL\n\n[![Build Status][badge-github]][github-build]\n[![Version][badge-version]][hexpm]\n[![Downloads][badge-downloads]][hexpm]\n[![License][badge-license]][github-license]\n\n> SSRF Protection in Elixir 🛡️\n\nSafeURL is a library that aids developers in protecting against a class of vulnerabilities\nknown as Server Side Request Forgery. It does this by validating a URL against a configurable\nallow or block list before making an HTTP request.\n\nSee the [Documentation][docs] on HexDocs.","ref":"readme.html"},{"type":"extras","title":"Installation - SafeURL","doc":"To get started, add `safeurl` to your project dependencies in `mix.exs`. Optionally, you may\nalso add [`HTTPoison`][lib-httpoison] to your dependencies for making requests directly\nthrough SafeURL:\n\n```elixir\ndef deps do\n  [\n    {:safeurl, \"~> 1.0\"},\n    {:httpoison, \"~> 2.2\"},  # Optional\n  ]\nend\n```\n\nTo use SafeURL with your favorite HTTP Client, see the [HTTP Clients][readme-http] section.","ref":"readme.html#installation"},{"type":"extras","title":"Usage - SafeURL","doc":"`SafeURL` blocks private/reserved IP addresses are by default, and users can add additional\nCIDR ranges to the blocklist, or alternatively allow specific CIDR ranges to which the\napplication is allowed to make requests.\n\nYou can use `allowed?/2` or `validate/2` to check if a URL is safe to call. If you have the\n[`HTTPoison`][lib-httpoison] application available, you can also call `get/4` which will\nvalidate the host automatically before making a web request, and return an error otherwise.\n\n```elixir\niex> SafeURL.allowed?(\"https://includesecurity.com\")\ntrue\n\niex> SafeURL.validate(\"http://google.com/\", schemes: ~w[https])\n{:error, :unsafe_scheme}\n\niex> SafeURL.validate(\"http://230.10.10.10/\")\n{:error, :unsafe_reserved}\n\niex> SafeURL.validate(\"http://230.10.10.10/\", block_reserved: false)\n:ok\n\n# When HTTPoison is available:\n\niex> SafeURL.HTTPoison.get(\"https://10.0.0.1/ssrf.txt\")\n{:error, :unsafe_reserved}\n\niex> SafeURL.HTTPoison.get(\"https://google.com/\")\n{:ok, %HTTPoison.Response{...}}\n```","ref":"readme.html#usage"},{"type":"extras","title":"Configuration - SafeURL","doc":"`SafeURL` can be configured to customize and override validation behaviour by passing the\nfollowing options:\n\n- `:block_reserved` - Block reserved/private IP ranges. Defaults to `true`.\n\n- `:blocklist` - List of CIDR ranges to block. This is additive with `:block_reserved`.\n  Defaults to `[]`.\n\n- `:allowlist` - List of CIDR ranges to allow. If specified, blocklist will be ignored.\n  Defaults to `[]`.\n\n- `:schemes` - List of allowed URL schemes. Defaults to `[\"http, \"https\"]`.\n\n- `:dns_module` - Any module that implements the `SafeURL.DNSResolver` behaviour.\n  Defaults to `DNS` from the [`:dns`][lib-dns] package.\n\n- `:detailed_error` - Return specific error if validation fails. If set to `false`, `validate/2` will return `{:error, :restricted}` regardless of the reason. Defaults to `true`.\n\nThese options can be passed to the function directly or set globally in your `config.exs`\nfile:\n\n```elixir\nconfig :safeurl,\n  block_reserved: true,\n  blocklist: ~w[100.0.0.0/16],\n  schemes: ~w[https],\n  dns_module: MyCustomDNSResolver\n```\n\nFind detailed documentation on [HexDocs][docs].","ref":"readme.html#configuration"},{"type":"extras","title":"HTTP Clients - SafeURL","doc":"While SafeURL already provides a convenient [`SafeURL.HTTPoison.get/3`][docs-get] method to validate hosts\nbefore making GET HTTP requests, you can also write your own wrappers, helpers or\nmiddleware to work with the HTTP Client of your choice.","ref":"readme.html#http-clients"},{"type":"extras","title":"HTTPoison - SafeURL","doc":"For [HTTPoison][lib-httpoison], you can create a wrapper module that validates hosts\nbefore making HTTP requests:\n\n```elixir\ndefmodule CustomClient do\n  def request(method, url, body, headers \\\\ [], opts \\\\ []) do\n    {safeurl_opts, opts} = Keyword.pop(opts, :safeurl, [])\n\n    with :ok <- SafeURL.validate(url, safeurl_opts) do\n      HTTPoison.request(method, url, body, headers, opts)\n    end\n  end\n\n  def get(url, headers \\\\ [], opts \\\\ []),        do: request(:get, url, \"\", headers, opts)\n  def post(url, body, headers \\\\ [], opts \\\\ []), do: request(:post, url, body, headers, opts)\n  # ...\nend\n```\n\nAnd you can use it as:\n\n```elixir\niex> CustomClient.get(\"http://230.10.10.10/data.json\", [], safeurl: [block_reserved: false], recv_timeout: 500)\n{:ok, %HTTPoison.Response{...}}\n```","ref":"readme.html#httpoison"},{"type":"extras","title":"Tesla - SafeURL","doc":"For [Tesla][lib-tesla], `SafeURL` provides a helper middleware out-of-the-box, which you can plug anywhere you're using `Tesla`:\n\n```elixir\ndefmodule DocumentService do\n  use Tesla\n\n  plug Tesla.Middleware.BaseUrl, \"https://document-service/\"\n  plug Tesla.Middleware.JSON\n  plug SafeURL.TeslaMiddleware, schemes: ~w[https], allowlist: [\"10.0.0.0/24\"]\n\n  def fetch(id) do\n    get(\"/documents/#{id}\")\n  end\nend\n```","ref":"readme.html#tesla"},{"type":"extras","title":"Custom DNS Resolver - SafeURL","doc":"In some cases you might want to use a custom strategy for DNS resolution. You can do so by\npassing your own implementation of [`SafeURL.DNSResolver`][docs-dns] in the global or local\nconfig.\n\nExample use-cases of this are:\n\n- Using a specific DNS server\n- Avoiding network access in specific environments\n- Mocking DNS resolution in tests\n\nYou can do so by implementing `DNSResolver`:\n\n```elixir\ndefmodule TestDNSResolver do\n  @behaviour SafeURL.DNSResolver\n\n  @impl true\n  def resolve(\"google.com\"), do: {:ok, [{192, 168, 1, 10}]}\n  def resolve(\"github.com\"), do: {:ok, [{192, 168, 1, 20}]}\n  def resolve(_domain),      do: {:ok, [{192, 168, 1, 99}]}\nend\n```\n\n```elixir\nconfig :safeurl, dns_module: TestDNSResolver\n```\n\nFor more examples, see [`SafeURL.DNSResolver`][docs-dns] docs.","ref":"readme.html#custom-dns-resolver"},{"type":"extras","title":"Contributing - SafeURL","doc":"- [Fork][github-fork], Enhance, Send PR\n- Lock issues with any bugs or feature requests\n- Implement something from Roadmap\n- Spread the word :heart:","ref":"readme.html#contributing"},{"type":"extras","title":"About - SafeURL","doc":"SafeURL is officially maintained by the team at [Slab][slab]. It was originally created by Nick Fox at\n[Include Security][includesecurity].\n\n \n\n[badge-github]: https://github.com/slab/safeurl-elixir/actions/workflows/ci.yml/badge.svg\n[badge-version]: https://img.shields.io/hexpm/v/safeurl.svg\n[badge-license]: https://img.shields.io/hexpm/l/safeurl.svg\n[badge-downloads]: https://img.shields.io/hexpm/dt/safeurl.svg\n[hexpm]: https://hex.pm/packages/safeurl\n[github-build]: https://github.com/slab/safeurl-elixir/actions/workflows/ci.yml\n[github-license]: https://github.com/slab/safeurl-elixir/blob/main/LICENSE\n[github-fork]: https://github.com/slab/safeurl-elixir/fork\n[slab]: https://slab.com/\n[includesecurity]: https://github.com/IncludeSecurity\n[readme-http]: #http-clients\n[docs]: https://hexdocs.pm/safeurl\n[docs-get]: https://hexdocs.pm/safeurl/SafeURL.html#get/4\n[docs-dns]: https://hexdocs.pm/safeurl/SafeURL.DNSResolver.html\n[lib-dns]: https://github.com/tungd/elixir-dns\n[lib-tesla]: https://github.com/elixir-tesla/tesla\n[lib-httpoison]: https://github.com/edgurgel/httpoison","ref":"readme.html#about"},{"type":"extras","title":"Migrating from 0.3 to 1.0","doc":"# Migrating from 0.3 to 1.0","ref":"migrating_to_1-0.html"},{"type":"extras","title":"Replace `SafeURL.get/4` with `SafeURL.HTTPoison.get/3` - Migrating from 0.3 to 1.0","doc":"`SafeURL.get/4` has been replaced with `SafeURL.HTTPoison.get/3`. It is now a\ndrop-in replacement for `HTTPoison.get/3` and thus does not support passing\n`SafeURL` options through function arguments.","ref":"migrating_to_1-0.html#replace-safeurl-get-4-with-safeurl-httpoison-get-3"},{"type":"extras","title":"Account for new error messages or use `:detailed_error` option - Migrating from 0.3 to 1.0","doc":"`SafeURL.validate/2` now returns a more specific error by default:\n\n```elixir\niex> SafeURL.validate(\"http://localhost\")\n{:error, :unsafe_reserved}\niex> SafeURL.validate(\"http://google.com\", schemes: [:https])\n{:error, :unsafe_scheme}\n```\n\nYou can use the `:detailed_error` configuration option to restore the previous\nbehavior and get the generic `{:error, :restricted}` error:\n\n```elixir\niex> SafeURL.validate(\"http://localhost\", detailed_error: false)\n{:error, :restricted}\niex> Application.put_env(:safeurl, :detailed_error, false)\n:ok\niex> SafeURL.validate(\"http://localhost\")\n{:error, :restricted}\n```","ref":"migrating_to_1-0.html#account-for-new-error-messages-or-use-detailed_error-option"},{"type":"extras","title":"Change modules implementing `SafeURL.DNSResolver` behaviour - Migrating from 0.3 to 1.0","doc":"If you have a module that implements `SafeURL.DNSResolver` behaviour, note that\n`DNSResolver.resolve/1` signature has changed. Specifically, the ok tuple should now\nalways return a list of IPs. A single IP is not a valid return type:\n\n```elixir\n# DNS is the default implementation\niex> DNS.resolve(\"wikipedia.org\")\n{:ok, [{198, 35, 26, 96}]}\n```","ref":"migrating_to_1-0.html#change-modules-implementing-safeurl-dnsresolver-behaviour"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.35.1"}}